/**
 * getkey.C: This file contains code for the ChanServ GETKEY functions.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005 William Pitcock, et al.
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: getkey.C,v 1.8 2007/09/22 14:27:27 pippijn Exp $
 */

#include "atheme.h"
#include <ermyth/module.h>
#include <account/mychan.h>
#include <account/chanacs.h>

static char const rcsid[] = "$Id: getkey.C,v 1.8 2007/09/22 14:27:27 pippijn Exp $";

REGISTER_MODULE ("chanserv/getkey", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void cs_cmd_getkey (sourceinfo_t *si, int parc, char *parv[]);

command_t const cs_getkey = { "GETKEY", N_("Returns the key (+k) of a channel."), AC_NONE, 1, cs_cmd_getkey };

E cmdvec cs_cmdtree;
E helpvec cs_helptree;

bool
_modinit (module *m)
{
  cs_cmdtree << cs_getkey;
  help_addentry (cs_helptree, "GETKEY", "help/chanserv/getkey", NULL);

  return true;
}

void
_moddeinit ()
{
  cs_cmdtree >> cs_getkey;
  help_delentry (cs_helptree, "GETKEY");
}

static void
cs_cmd_getkey (sourceinfo_t *si, int parc, char *parv[])
{
  char *chan = parv[0];
  mychan_t *mc;

  if (!chan)
    {
      command_fail (si, fault::needmoreparams, STR_INSUFFICIENT_PARAMS, "GETKEY");
      command_fail (si, fault::needmoreparams, _("Syntax: GETKEY <#channel>"));
      return;
    }

  mc = mychan_t::find (chan);
  if (!mc)
    {
      command_fail (si, fault::nosuch_target, _("\2%s\2 is not registered."), chan);
      return;
    }

  if (mc->find_metadata ("private:close:closer"))
    {
      command_fail (si, fault::noprivs, _("\2%s\2 is closed."), chan);
      return;
    }

  if (!chanacs_source_has_flag (mc, si, CA_INVITE))
    {
      command_fail (si, fault::noprivs, _("You are not authorized to perform this operation."));
      return;
    }

  if (!mc->chan)
    {
      command_fail (si, fault::nosuch_target, _("\2%s\2 is currently empty."), mc->name);
      return;
    }

  if (!mc->chan->key)
    {
      command_fail (si, fault::nosuch_key, _("\2%s\2 is not keyed."), mc->name);
      return;
    }
  logcommand (si, CMDLOG_GET, "%s GETKEY", mc->name);
  command_success_string (si, mc->chan->key, _("Channel \2%s\2 key is: %s"), mc->name, mc->chan->key);
}

/* vim:cinoptions=>s,e0,n0,f0,{0,}0,^0,=s,ps,t0,c3,+s,(2s,us,)20,*30,gs,hs
 * vim:ts=8
 * vim:sw=8
 * vim:noexpandtab
 */
